/*************************************************************************
 *  GemForm.C : Fonctions de gestion des formulaires en fenetre.         *
 *************************************************************************/

#include "WindGem.h"
/*-=< Inclusion des descriptions des objets UserDefs. >=-----------------*/
#include "user.c"

/*-=< Section de declaration des fonctions privees >=--------------------*/
static void coordonnees (void);

/* Gestion des listes d'objets */
Wind *ObjListe(int numObj);
Wind *TopListe(int handle);

/* Fonctions privees, internes a gemform */
static void WindFormDo(Wind *wind, int evnt);
static int do_keybd(OBJECT *tree, int objc);
static void gere_boutons(void);
static void gere_clavier(void);
static void EvntSpe (int *evnt);

/* Fonctions externes provenant des autres modules */
extern void AjouteListe(int type, int numObj, int mode, char *title, GRECT pos, void *ptr_var, void (*fonct)(int evnt));
extern void WindTextDo (Wind *wind, int evnt);
extern void WindTextSliders (Wind *wind);
extern int stdkey (unsigned char *k);
extern int parent (OBJECT *adr, int object);
extern int m_title (OBJECT *adresse, int option);
extern void DrawIcone (int handle, GRECT *zone);
extern void AfficheAide (Wind *wind, int objet);


OBJECT *AdrDesk;							/* Adresse du bureau                       */
OBJECT *AdrMenu;							/* Adresse du menu                         */
int buff[8];									/* Buffer evenement                        */
System *Sys;									/* Variable Systeme                        */
DTA dtabuffer;								/* Tampon DTA                              */
Wind LstFen[NBWIND];					/* Liste des fenetres (60 fenetres max.)   */

/*************************************************************************
 * Fonctions de gestion de l'application.                                *
 *************************************************************************/
/*-----------------------------------------------------------------------*
 * Fonction qui fixe les coordonnes des objets USerDefs.                *
 *-----------------------------------------------------------------------*/
static void coordonnees (void)
{
	int i, j, nb_tree;

	nb_tree = 2;	/* Un pour les Userdefs, un pour l'icone */
	for (i = 0 ; i < nb_tree ; i++) 	/* Pour chaque arbre du RSC intgr, */
	{
		j = 0;													/* et chaque objet,                  */
		do
		{	/* Tant qu'on n'est pas au dernier objet */
			rsrc_obfix (rs_trindex[i], j);		/* Adapter coordonnes           */
		} while (! (rs_trindex[i][j++].ob_flags & LASTOB));
	}
	Sys->AdrUser = rs_trindex[0];
	Sys->AdrIcone = rs_trindex[1];
	Sys->AdrAide = rs_trindex[2];
}

/*-----------------------------------------------------------------------*
 * Fonction d'initialisation de l'application.                           *
 *                                                                       *
 * fic_rsc	: Fichier .rsc de l'application                              *
 * desk			: numero de l'objet pour bureau                              *
 * gereDesk : fonction de gestion du bureau                              *
 * menu			: numero de l'objet MENU                                     *
 * gereMenu	: fonction de gestion du menu                                *
 *-----------------------------------------------------------------------*/
void AppInit(char *fic_rsc, int desk, void (*gereDesk)(int event), int menu, void (*gereMenu)(int event))
{
	int i, attributes[12];
	
	Sys = (System *)malloc(sizeof(System));

	Sys->AppId = appl_init();

	for (i = 1 ; i < 9 ; _GemParBlk.intin[i++] = 1);
	_GemParBlk.intin[0] = Getrez() + 2;			 /* on rcupre la rsolution  */
	_GemParBlk.intin[9] = 0;								 /* Couleur de Remplissage = 0 */
	_GemParBlk.intin[10]= 2;								 /* des coordonnes            */

	Sys->VdiHandle = graf_handle(&(Sys->Wchar), &(Sys->Hchar), &i, &i);
	Sys->Hchar++;	/* +1 pour eviter que les lignes ne se touchent */
	/* Ouverture station de travail virtuelle */
	v_opnvwk (_GemParBlk.intin, &(Sys->VdiHandle), _GemParBlk.intout);

	if (_GemParBlk.intout[13] < 16)
	{ /* Etablissement des couleurs 3D */
		Sys->color_3d1 = WHITE;														  /* color index 0 */
		Sys->color_3d2 = BLACK;	                            /* color index 1 */
	}
	else
	{
		Sys->color_3d1 = LWHITE;	                          /* color index 8 */
		Sys->color_3d2 = LBLACK;	                          /* color index 9 */
	}
	
	/* Coordonnes du bureau */
	wind_get (0, WF_WORKXYWH, &(Sys->Xdesk), &(Sys->Ydesk), &(Sys->Wdesk), &(Sys->Hdesk));

	Sys->Xres = _GemParBlk.intout[0];			/* Resolution en X du bureau     */
	Sys->Yres = _GemParBlk.intout[1];			/* Resolutino en Y du bureau     */

	vq_extnd (Sys->VdiHandle, TRUE, _GemParBlk.intout);
	Sys->Nplane = _GemParBlk.intout[4];		/* Nombre de plans de couleurs   */
	Sys->lastImg = 1000;									/* Der. num. pour fenetre image  */
	Sys->lastTxt = 2000;									/* Der. num. pour fenetre texte  */

	vqt_attributes(Sys->VdiHandle, attributes);
	/* Ligne de base des caractres + 1 pour viter que la premire ligne 
	de texte ne touche le haut de la fentre. */
	Sys->baseline = 1 + attributes[7]; 		

	if (NOT rsrc_load(fic_rsc))
	{
		char mes[60];

		sprintf(mes,"[2][| Le Fichier Ressource |    %s    | n'a pu tre trouv.][ Quitter ]",fic_rsc);
		form_alert(1,mes);
	  wind_set (0, WF_NEWDESK, 0, 0); 					/* Rendre le bureau        */
		v_clsvwk(Sys->VdiHandle);									/* Efface l'ecran          */
		free (Sys);
		appl_exit();															/* Fermer application      */
	}
	else
	{
		/* Init. des 60 fenetres */
		for (i=0; i<NBWIND; i++)
		{
			LstFen[i].handle	= BLANK;
			LstFen[i].numObj	= ZERO;
			LstFen[i].type		= WTYPNONE;
			LstFen[i].mode		= WNORM;
			LstFen[i].fonct		= (void *)NULL;
			LstFen[i].w_title	= (char *)NULL;
			LstFen[i].iconified = FALSE;
		}

		/* Mise en place du Bureau *******************************************/
		LstFen[0].type		= WTYPDESK;
		LstFen[0].handle	= 0;

		/* Si bureau , le creer en tant que fenetre 0 */
		if (desk != BLANK)
		{
			rsrc_gaddr (0, desk, &AdrDesk);			/* Demander adresse bureau */
  		AdrDesk->ob_x = Sys->Xdesk;		     	/* Mettre le bureau aux    */
		  AdrDesk->ob_y = Sys->Ydesk;		   		/* bonnes dimensions.      */
		  AdrDesk->ob_width = Sys->Wdesk;
	  	AdrDesk->ob_height = Sys->Hdesk;
	  	wind_set (0, WF_NEWDESK, AdrDesk, 0);/* Fixer le nouveau bureau */
																							/* et l'afficher.          */
		  form_dial (FMD_FINISH, 0, 0, 0, 0, Sys->Xdesk, Sys->Ydesk, Sys->Wdesk, Sys->Hdesk); 
			LstFen[0].numObj	= desk;
			LstFen[0].fonct	= gereDesk;
		}
		else
			AdrDesk  = (OBJECT *)NULL;

		/* Mise en place du Menu *********************************************/
		if (menu != BLANK)
		{
			rsrc_gaddr (0, menu, &AdrMenu);					/* Demander adresse menu   */
		  menu_bar (AdrMenu, 1);  		    				/* Afficher le menu        */
			Sys->menu = gereMenu;
		}
		else
			AdrMenu  = (OBJECT *)NULL;

		Sys->AdrObjet = (OBJECT *)NULL;				/* Objet pour gestion interne  */
		Sys->objet	= ZERO;										/* idem */
		Sys->pos		= ZERO;										/* idem */

		/* Popup lors de l'appui sur bouton droit */
		Sys->popMenu = BLANK;
		Sys->popup = (void *)NULL;
	}

	Fsetdta (&dtabuffer);											 /* Fixer adresse tampon DTA */

	coordonnees();									 /* Mise en place des boutons userdefs */

	graf_mouse(ARROW, 0);

}

/*-----------------------------------------------------------------------*
 * Fonction d'initialisation du popup menu appel lors de l'appui sur le *
 * bouton droit.                                                         *
 *                                                                       *
 * numObj 	: Numro de l'objet pour popup                               *
 * gerePopup: Fonction de gestion du popup menu                          *
 *-----------------------------------------------------------------------*/
void PopMenu(int numObj, void (*gerePopup)(int option))
{
	if (numObj > 0 && gerePopup != (void *)NULL)
	{
		Sys->popMenu = numObj;
		Sys->popup = gerePopup;
	}
		
}

/*-----------------------------------------------------------------------*
 * Fermeture de l'application                                            *
 *-----------------------------------------------------------------------*/
void AppExit(void)
{
	int i;

	/* Destruction de toutes les fenetres */
	for (i=1; i<NBWIND; i++)
		if (LstFen[i].type != WTYPNONE)
			WindDelete(LstFen[i].numObj);

	if (AdrMenu != (OBJECT *)NULL)
	  menu_bar (AdrMenu, 0); 			 				      /* Virer la barre de menu  */

  wind_set (0, WF_NEWDESK, 0, 0); 						/* Rendre le bureau        */
	rsrc_free();																/* Liberer espace RSC      */
	v_clsvwk(Sys->VdiHandle);										/* Efface l'ecran          */
	free (Sys);
	appl_exit();																/* Fermer application      */
}

/*************************************************************************
 * Fonctions de gestion de la liste des fenetres.                        *
 * Validation            : 20/04/1996                                    *
 *************************************************************************/

/*-----------------------------------------------------------------------*
 * Retourne l'objet Wind associe a l'objet <numObj>, NULL sinon          *
 *-----------------------------------------------------------------------*/
Wind *ObjListe(int numObj)
{
	int i = 1;

	while (i < NBWIND)
	{
		if (LstFen[i].numObj == numObj)
			return &LstFen[i];
		i++;
	}
	return WIND_NULL;
}

/*-----------------------------------------------------------------------*
 * Retourne l'objet de handle <handle> donne, NULL sinon                 *
 *-----------------------------------------------------------------------*/
Wind *TopListe(int handle)
{
	int i = 1;

	while (i < NBWIND)
	{
		if (LstFen[i].handle == handle)
			return &LstFen[i];
		i++;
	}
	return WIND_NULL;
}

/*-----------------------------------------------------------------------*
 * Initialisation des fenetres formulaires.                              *
 *                                                                       *
 * numObj     : index de l'objet a mettre en fenetre                     *
 * inf_x      : position X de la fenetre au depart                       *
 * inf_y      : position Y de la fenetre au depart                       *
 * title      : titre du formulaire                                      *
 * edit       : numero du 1er champ editable                             *
 * fonct      : fonction de gestion du formulaire                        *
 *                                                                       *
 * Derniere modification : 22/06/1996                                    *
 *-----------------------------------------------------------------------*/
void WindFormInit(int numObj, int mode, int inf_x, int inf_y, char *title, int edit, void (*fonct)(int evnt))
{
	int wx, wy, ww, wh;
	int offset = 0, inf_flag = 0;
	OBJECT *adrForm;
	GRECT pos;
	WindForm *ptr_dial = (WindForm *)malloc(sizeof(WindForm));

	rsrc_gaddr(R_TREE, numObj, &adrForm);
	ptr_dial->adr_form = adrForm;
	if (adrForm->ob_state & OUTLINED)	/* Ce decalage est invariable !!! */
		ptr_dial->offset = offset = 3;
	else
	{
		if (global[1] > 1) /* Si Multitache */
			ptr_dial->offset = 1; /* surtout pour Magic 4 */
		else
			ptr_dial->offset = 0;
	}		

	form_center(adrForm, &wx, &wy, &ww, &wh);
	wind_calc(WC_BORDER, FW_ATTRIB, wx, wy, ww, wh, &wx, &wy, &ww, &wh);
	if (inf_x != 0 || inf_y != 0)
	{	/* si valeurs lues dans fichier .INF ... */
		wx = inf_x; wy = inf_y;
		inf_flag = TRUE;
	}
	pos.g_x = wx;
	pos.g_y = wy;
	pos.g_w = ww;
	pos.g_h = wh;
	ptr_dial->edit			= edit;
	ptr_dial->edit_objc = edit;
	ptr_dial->edit_pos 	= 0;
	/* Gestion de l'aide en ligne */
	ptr_dial->aide			= (AIDE *)NULL;
	
	if (inf_flag)
	{	/* ... alors recaler le formulaire sur la fenetre. */
		wind_calc(WC_WORK, FW_ATTRIB, wx, wy, ww, wh, &wx, &wy, &ww, &wh);
		adrForm->ob_x = wx + offset;
		adrForm->ob_y = wy + offset;
	}
	set_user(adrForm);							/* Etablissement des boutons USERDEFS. */
	
	AjouteListe (WTYPFORM, numObj, mode, title, pos, (void *)ptr_dial, fonct);
}

/*-----------------------------------------------------------------------*
 * Ouverture d'une fenetre.                                              *
 *                                                                       *
 * numObj			: numero de l'objet a afficher.                            *
 *                                                                       *
 * Derniere modification : 26/04/1996                                    *
 *-----------------------------------------------------------------------*/
void WindOpen(int numObj)
{
	int attrib;
	Wind *wind;
	OBJECT *ptr_objc;
	WindForm *ptr_dial;

	wind = ObjListe(numObj);

	if (wind->type == WTYPFORM)
	{
		if (wind->mode != WMODAL)
			attrib = FW_ATTRIB;
		else
			attrib = NAME;
	}
	else if (wind->type == WTYPGRAP)
		attrib = GW_ATTRIB;
	else
		attrib = TW_ATTRIB;

	if (wind->handle != BLANK)
		wind_set(wind->handle, WF_TOP);
	else
	{
		if ((wind->handle = wind_create(attrib, wind->Xpos, wind->Ypos, wind->Wpos, wind->Hpos)) != 0)
		{
			wind_set(wind->handle, WF_NAME, wind->w_title);
			graf_growbox(0, 0, 0, 0, wind->Xpos, wind->Ypos, wind->Wpos, wind->Hpos);
			wind_open(wind->handle, wind->Xpos, wind->Ypos, wind->Wpos, wind->Hpos);

			switch(wind->type)
			{
				case WTYPFORM :
					ptr_dial = wind->cont.dialog;
					ptr_dial->edit_objc = ptr_dial->edit;
					ptr_objc = &(ptr_dial->adr_form[ptr_dial->edit]);
					if ((ptr_objc->ob_flags & EDITABLE) != 0)
						ptr_dial->edit_pos = (int)strlen((ptr_objc->ob_spec.tedinfo)->te_ptext);
					/* Envoi de l'evenement OPEN a la fenetre */
					(*wind->fonct)(EV_OPEN);
					break;
				case WTYPTEXT :
					WindTextSliders (wind);
					break;
			}
			/* Si fenetre modale, on bloque les evenements MU_MESSAG */
			if (wind->mode == WMODAL)
			{
				Sys->lastMode = WMODAL;
				wind_update(3);
			}
		}
		else
		{
			form_alert(1, "[3][   Il n'y a plus de| fentres disponibles !"
										"| Fermez une fentre que|  vous n'utilisez pas.][Confirmer]");
		}
	}
}

/*-----------------------------------------------------------------------*
 * Fermeture d'une fenetre.                                              *
 *                                                                       *
 * numObj  : Numero de la fenetre                                        *
 *                                                                       *
 * Derniere modification : 25/04/1996                                    *
 *-----------------------------------------------------------------------*/
void WindClose(int numObj)
{
	Wind *wind;

	wind = ObjListe(numObj);

	/* Si fenetre modale, on reautorise les evenements MU_MESSAG */
	if (wind->mode == WMODAL)
	{
		Sys->lastMode = WNORM;
		wind_update(2);
	}
	wind_close(wind->handle);
	wind_delete(wind->handle);
	graf_shrinkbox(0, 0, 0, 0, wind->Xpos, wind->Ypos, wind->Wpos, wind->Hpos);
	wind->handle 	= BLANK;
}

/*-----------------------------------------------------------------------*
 * Destruction d'une fenetre.                                            *
 *                                                                       *
 * numObj  : Numero de la fenetre                                        *
 *                                                                       *
 * Derniere modification : 25/04/1996                                    *
 *-----------------------------------------------------------------------*/
void WindDelete(int numObj)
{
	Wind *wind;

	wind = ObjListe(numObj);

	/* Si fenetre ouverte, on la ferme */
	if (wind->handle != BLANK)
	{
		wind_close(wind->handle);
		wind_delete(wind->handle);
		graf_shrinkbox(0, 0, 0, 0, wind->Xpos, wind->Ypos, wind->Wpos, wind->Hpos);
	}

	if (wind->type == WTYPFORM)
	{
		WindForm *ptr_dial = wind->cont.dialog;
		AIDE *aide = ptr_dial->aide;
		
		if (ptr_dial->aide != (AIDE *)NULL)
		{
			free (aide->txt);
			free (aide);
		}
		free (wind->cont.dialog);
	}
	else
	{
		WindText *ptr_txt = wind->cont.text;
		free (ptr_txt->lignes);
		free (wind->cont.text);
	}
	wind->handle 	= BLANK;
	wind->numObj	= ZERO;
	wind->mode		= WNORM;
	wind->type 		= WTYPNONE;
	wind->fonct		= (void *)NULL;
	free (wind->w_title);
}


/*************************************************************************
 * fonctions de gestion des evenements                                   *
 *************************************************************************/

/*-----------------------------------------------------------------------*
 * Fonction de redessin d'un objet d'un formulaire en fentre.           *
 *                                                                       *
 * arbre 	: numro de l'arbre d'objets                                   *
 * objet  : objet  redessinner                                          *
 *                                                                       *
 * Auteur : ..........                                                   *
 *-----------------------------------------------------------------------*/
void WindDraw(int arbre, int objet)
{
	GRECT r;
	int dummy;
	int top;
  
	Wind *wind;
	WindForm *ptr_form;
  
  wind = ObjListe(arbre);
  ptr_form = wind->cont.dialog;
  
  if ( wind->handle > 0)  /* la fenetre existe ? */
  {
    wind_update(BEG_UPDATE);
    wind_get(0, WF_TOP, &top, &dummy, &dummy, &dummy);
    if ( wind->handle == top) /* si topped, on optimise... */        
      objc_draw(ptr_form->adr_form, objet, MAX_DEPTH, wind->Xpos,
        wind->Ypos, wind->Wpos,wind->Hpos);
        
    else
    {
	    wind_get(wind->handle, WF_FIRSTXYWH, &r.g_x, &r.g_y, &r.g_w, &r.g_h);
      while (r.g_w && r.g_h)
      {
        objc_draw(ptr_form->adr_form, objet, MAX_DEPTH, r.g_x, r.g_y,
                              r.g_w, r.g_h);
  		  wind_get(wind->handle, WF_NEXTXYWH, &r.g_x, &r.g_y, &r.g_w, &r.g_h);
      }
    }
    wind_update(END_UPDATE);
  }
}

/*-----------------------------------------------------------------------*
 * Fonction de gestion des formulaires en fenetre                        *
 *                                                                       *
 * wind 	: objet Wind a traiter                                         *
 * event  : evenement a traiter                                          *
 *                                                                       *
 * Derniere modification : 21/04/1996                                    *
 *-----------------------------------------------------------------------*/
static void WindFormDo(Wind *wind, int evnt)
{
	GRECT r, rd;
	int wx, wy, ww, wh;
	int dialog = TRUE, object, init_field = FALSE;
	WindForm *ptr_dial = wind->cont.dialog;
	
	if (evnt & MU_MESAG)
	{
		switch(buff[0])
		{
			case WM_REDRAW :
				rd.g_x = buff[4]; rd.g_y = buff[5];
				rd.g_w = buff[6]; rd.g_h = buff[7];
				if (wind->iconified == TRUE)
				{
					DrawIcone (wind->handle, &rd);
				}
				else
				{
					if (ptr_dial->edit_objc)
					{	/* si champ editable et ....*/
						wind_get(0, WF_TOP, &wx, &wy, &ww, &wh);
						if (wind->handle == wx)
						{ /*...si fenetre au 1 plan -> redessin total */
							wind_get(wind->handle, WF_WORKXYWH, &rd.g_x, &rd.g_y, &rd.g_w, &rd.g_h);
							init_field = EDEND;
						}
					}
					wind_update(BEG_UPDATE);
					wind_get(wind->handle, WF_FIRSTXYWH, &r.g_x, &r.g_y, &r.g_w, &r.g_h);
					while (r.g_w && r.g_h)
					{
						if (rc_intersect(&rd, &r))
							objc_draw(ptr_dial->adr_form, 0, 8, r.g_x, r.g_y, r.g_w, r.g_h);
						wind_get(wind->handle, WF_NEXTXYWH, &r.g_x, &r.g_y, &r.g_w, &r.g_h);
					}
					wind_update(END_UPDATE);
					if (ptr_dial->edit_objc && init_field)
						objc_edit(ptr_dial->adr_form, ptr_dial->edit_objc, 0, ptr_dial->edit_pos, init_field, &(ptr_dial->edit_pos));
				}
				break;

			case WM_TOPPED :
				wind_set(wind->handle, WF_TOP);
				if (ptr_dial->edit_objc && global[1] > 1)
				{
					wind_get(buff[3], WF_WORKXYWH, &wx, &wy, &ww, &wh);
					form_dial(FMD_FINISH, 0,0,0,0, wx, wy, ww, wh);				
				}
				break;

			case WM_CLOSED : 
				/* Envoi de l'evenement fermeture a l'utilisateur pour trtmt */
				(*wind->fonct)(EV_CLOSE);
				WindClose(wind->numObj);
				break;

			case WM_MOVED :
				wind_set(wind->handle, WF_CURRXYWH, buff[4], buff[5], buff[6], buff[7]);
				if (wind->iconified == FALSE)
				{
					wind->Xpos = buff[4];
					wind->Ypos = buff[5];
					wind_get(wind->handle, WF_WORKXYWH, &wx, &wy, &ww, &wh);
					ptr_dial->adr_form->ob_x = wx + ptr_dial->offset;
					ptr_dial->adr_form->ob_y = wy + ptr_dial->offset;
				}
				break;
				
			case WM_ICONIFY :
				graf_shrinkbox(buff[4], buff[5], buff[6], buff[7], wind->Xpos, wind->Ypos, wind->Wpos, wind->Hpos);
				wind_set(wind->handle, WF_ICONIFY, buff[4], buff[5], buff[6], buff[7]);
				wind->iconified = TRUE;
				break;
				
			case WM_UNICONIFY :
				graf_growbox( wind->Xpos, wind->Ypos, wind->Wpos, wind->Hpos, buff[4], buff[5], buff[6], buff[7]);
				wind_set(wind->handle, WF_UNICONIFY, buff[4], buff[5], buff[6], buff[7]);
				wind->iconified = FALSE;
				break;
		}
	}
	else if (evnt == EV_HELP)
	{
/* Gestion du bouton HELP, enfin pas encore... */		
	}
}

/*-----------------------------------------------------------------------*
 * Gestion du clavier.                                                   *
 *                                                                       *
 * tree : arbre d'objet a gerer                                          *
 * objc : numero d'objet dans l'arbre                                    *
 *                                                                       *
 * Derniere modification : 21/04/1996                                    *
 *-----------------------------------------------------------------------*/
static int do_keybd(OBJECT *tree, int objc)
{
	int dialog, len, index, key_short, pos;
	char *ed_text;
	unsigned char dum;
	OBJECT *ptr_objc;

	dialog = form_keybd(tree, objc, objc, Sys->key, &(Sys->new_objc), &(Sys->key));
	if (dialog)
	{
		if (Sys->key == 0)
		{
			if (Sys->new_objc != 0)
			{
				ptr_objc = &tree[Sys->new_objc];
				Sys->new_pos = (int) strlen((ptr_objc->ob_spec.tedinfo)->te_ptext);
			}
			else
				Sys->new_pos = 0;
		}
		else
		{
			if (objc)
			{
				pos = Sys->new_pos;
				
				ptr_objc = &tree[objc];
				ed_text = (ptr_objc->ob_spec.tedinfo)->te_ptext;
				len = (int) strlen(ed_text);
				switch (Sys->key)
				{
					case 0x7300 :		/* Control <- : Curseur en debut de Champ */
						pos = 0;
						Sys->key = 0;
						break;
					case 0x7400 :		/* Control -> : Curseur en fin de Champ */
						pos = len;
						Sys->key = 0;
						break;
					case 0x4b34 :		/* SHIFT <- : Saute un mot	*/
						while (pos)
						{
							pos--;
							if (ed_text[pos] == ' ')
								break;
						}
						Sys->key = 0;
						break;
					case 0x4d36 :		/* SHIFT -> : Saute un mot	*/
						while (pos < len)
						{
							pos++;
							if (ed_text[pos] == ' ')
								break;
						}
						if (pos < len)
							pos++;
						Sys->key = 0;
						break;
				}
				Sys->new_pos = pos;
			}
			if (Sys->key != 0 && Sys->kbd == K_ALT)
			{	/* Raccourcis claviers	*/
				key_short = stdkey(&dum);
				index = 1;
				do
				{
					ptr_objc = &tree[index]; /* pointe sur l'objet a traiter */
					ed_text = 0;
					if (((ptr_objc->ob_state&DISABLED)==0) && ((ptr_objc->ob_type & 0x200) != 0))
					{ /* si l'objet n'est pas desactive	*/
						switch (ptr_objc->ob_type & 0xff)
						{
							case G_BUTTON :
								ed_text = ptr_objc->ob_spec.free_string;
								break;
							case G_USERDEF :
								ed_text = (char *)((ptr_objc->ob_spec.userblk)->ub_parm);
						}
						if (ed_text && (ed_text = strchr(ed_text, '[')) != 0)
						{
							if (key_short == toupper(ed_text[1]))
							{ /* Simuler un clic sur l'objet et retour : */
								Sys->key = 0;
								Sys->objet = index;
								form_button(tree, index, 1, &(Sys->new_objc));
								evnt_timer(100,0);
								return -1;
							}
						}
					}
					index ++; /* Prepare l'objet suivant ... */
				/* ... sauf si l'objet actuel est le dernier : */
				} while((ptr_objc->ob_flags & LASTOB) == 0);
			}
		}
	}
	return dialog;
}

/*-----------------------------------------------------------------------*
 * Fonction de remplacement du Form_do du GEM.                           *
 *                                                                       *
 * Derniere modification : 17/05/1996                                    *
 *-----------------------------------------------------------------------*/
int EventMulti (void)
{
	static int xmouse = 0, ymouse = 0;
	int evnt;								/* Type d'vnement                            */
	int top, dummy;
	Wind *wind;							/* Pointeur pour element de liste de fenetre   */
	char mes[80];
	WindForm *ptr_dial;
		
	Sys->objet = ZERO;			/* Mise  zro avant de commencer              */

	while (TRUE) 						/* BOUCLE "SANS FIN"                           */
	{ /* Surveillance des vnements Clavier, Clic, Message et Timer       */
		evnt = evnt_multi ((MU_MESAG|MU_BUTTON|MU_KEYBD|MU_TIMER),
											 258, 3, 0,	/* au lieu de 2,1,1 pour gerer les deux boutons */
											 ZERO, ZERO, ZERO, ZERO, ZERO, ZERO, ZERO, ZERO, ZERO, ZERO,
											 buff, 500, 0,
											 &(Sys->mousex), &(Sys->mousey), &(Sys->mousek),
											 &(Sys->kbd), &(Sys->key), &(Sys->clik));

/*-----------------------------------------------------------------------*
 * Gestion des evenements Timer.                                         *
 *-----------------------------------------------------------------------*/
		if (evnt & MU_TIMER)
		{
			int mx, my;
			
			graf_mkstate (&mx, &my, &dummy, &dummy);
  		if (xmouse == mx && ymouse == my)
  		{
				wind_get (ZERO, WF_TOP, &top, &dummy, &dummy, &dummy);
				wind = TopListe(top);

				if (wind != WIND_NULL && wind->type == WTYPFORM && NOT wind->iconified)
				{
					ptr_dial = wind->cont.dialog;
					Sys->AdrObjet = ptr_dial->adr_form;
					Sys->objet = objc_find (Sys->AdrObjet, ROOT, MAX_DEPTH, Sys->mousex, Sys->mousey);
					AfficheAide (wind, Sys->objet);
				}
			}
			else
			{
  			xmouse = mx;
 				ymouse = my;
			}
		}
	
/*-----------------------------------------------------------------------*
 * Gestion des evenements clavier.                                       *
 *-----------------------------------------------------------------------*/
		if (evnt & MU_KEYBD)			
			gere_clavier();

/*-----------------------------------------------------------------------*
 * Gestion des evenements Message                                        *
 *-----------------------------------------------------------------------*/
 		if ((evnt & MU_MESAG) && buff[0] == MN_SELECTED && AdrMenu != (OBJECT *)NULL)
		{	/* Appel a la fonction de gestion du menu */
			(*Sys->menu)(buff[4]);
			menu_tnormal(AdrMenu, buff[3], 1);
		}
		else if (evnt & MU_MESAG)										 /* Si vnement message */
		{
			/* Recupere fenetre qui a recu le message */
			wind = TopListe(buff[3]);

			if (wind != WIND_NULL)
			{
				switch (wind->type)
				{
					case WTYPFORM :
						WindFormDo(wind, evnt);
						break;
					case WTYPTEXT :
						WindTextDo(wind, evnt);
						break;
				}			
			}
    }
/*-----------------------------------------------------------------------*
 * Gestion des evenements Bouton                                         *
 *-----------------------------------------------------------------------*/

		if (evnt & MU_BUTTON)
		{
			if (Sys->mousek == 2 && Sys->popMenu != BLANK)
				pop_up (BLANK, BLANK, Sys->popMenu);
			else
				gere_boutons();
		}

		return evnt;
	}
}

/*-----------------------------------------------------------------------*
 * Deselection d'un objet d'un arbre donne                               *
 *                                                                       *
 * arbre	 : Numero de l'arbre d'objets                                  *
 * numObjc : Numero de l'objet                                           *
 *-----------------------------------------------------------------------*/
void ObjcUnselect(int arbre, int numObjc)
{
	Wind *wind;
	WindForm *ptr_form;
	
	/* On recupere les info sur l'arbre d'objet */
	wind = ObjListe(arbre);
	ptr_form = wind->cont.dialog;
	
	objc_change(ptr_form->adr_form, numObjc, 0, wind->Xpos,
							wind->Ypos, wind->Wpos, wind->Hpos, 0, TRUE);
}

/*-----------------------------------------------------------------------*
 * Retourne le numero de l'objet EXIT qui vient d'etre selectionne       *
 *-----------------------------------------------------------------------*/
int GetObjet(void)
{
	return Sys->objet;
}

/*-----------------------------------------------------------------------*
 * Gestion des evenements boutons                                        *
 *                                                                       *
 * Derniere modification : 17/05/1996                                    *
 *-----------------------------------------------------------------------*/
static void gere_boutons(void)
{
	int top, obflags, obstate, dummy, i, j, edit;
	Wind *wind;
	WindForm *ptr_dial;

	if (wind_find (Sys->mousex, Sys->mousey))
	{	/* Chercher fentre de 1er plan */
		wind_get (ZERO, WF_TOP, &top, &dummy, &dummy, &dummy);
		wind = TopListe(top);

		if (wind != WIND_NULL && wind->type == WTYPFORM && NOT wind->iconified)
		{
			ptr_dial = wind->cont.dialog;
			Sys->AdrObjet = ptr_dial->adr_form;
		}
		else
		{	/* Si fenetre iconifiee ou fenetre images ou texte, on oublie l'evenement */
			return;
		}
	}
	else
		Sys->AdrObjet = AdrDesk;

	Sys->objet = objc_find (Sys->AdrObjet, ROOT, MAX_DEPTH, Sys->mousex, Sys->mousey); 

	if (Sys->objet > BLANK) 							/* Si on a cliqu sur un objet */
	{
		obflags = (Sys->AdrObjet)[Sys->objet].ob_flags;
		obstate = (Sys->AdrObjet)[Sys->objet].ob_state;

		if (NOT (obstate & DISABLED))		 /* Si l'objet n'est pas dsactiv */
		{
			if (obflags & EDITABLE)
			{ /* Desactiver curseur */
   	    objc_edit (Sys->AdrObjet, ptr_dial->edit_objc, 0, ptr_dial->edit_pos, EDEND, &(ptr_dial->edit_pos));
				/* Reactiver curseur */
 		  	objc_edit (Sys->AdrObjet, Sys->objet, 0, Sys->pos, EDINIT, &(Sys->pos));

				if (Sys->AdrObjet != AdrDesk)
				{
					ptr_dial->edit_objc	= Sys->objet;
					ptr_dial->edit_pos	= Sys->pos;
				}
			}
	
			if ((obflags & SELECTABLE) &&	(NOT (obflags & RBUTTON)))
			{	/* Si slectable simple, inverser l'etat de l'objet */
				(Sys->AdrObjet)[Sys->objet].ob_state ^= SELECTED; 

				objc_draw (Sys->AdrObjet, Sys->objet, MAX_DEPTH, 
								  (Sys->AdrObjet)->ob_x, (Sys->AdrObjet)->ob_y,
									(Sys->AdrObjet)->ob_width, (Sys->AdrObjet)->ob_height);

				evnt_timer(100,0);
	
				/* Appel  la fonction de gestion de la boite de dialogue */
				if (obflags & TOUCHEXIT || obflags & EXIT)
					(*wind->fonct)(MU_BUTTON);
			}
			else
			{
				if (NOT (obflags & TOUCHEXIT))	 /* Si ce n'est pas un TOUCHEXIT */
				{
					while (Sys->mousek)					 /* Attendre bouton souris relach */
						graf_mkstate (&dummy, &dummy, &(Sys->mousek), &dummy);
				}
			}

			if ((obflags & SELECTABLE) &&	(obflags & RBUTTON) && (NOT (obstate & SELECTED)))
			{	/* Si radio-bouton */
				j = Sys->objet;
				(Sys->AdrObjet)[Sys->objet].ob_state |= SELECTED;
	
				objc_draw (Sys->AdrObjet, Sys->objet, MAX_DEPTH,
									(Sys->AdrObjet)->ob_x, (Sys->AdrObjet)->ob_y, 
									(Sys->AdrObjet)->ob_width, (Sys->AdrObjet)->ob_height);

				i = parent (Sys->AdrObjet, j);			 /* Chercher le pre       */
				j = (Sys->AdrObjet)[i].ob_head; 		 /* Partir du 1 enfant... */
				i = (Sys->AdrObjet)[i].ob_tail; 		 /* jusqu'au dernier.      */
				do
				{
					if (((Sys->AdrObjet)[j].ob_flags & RBUTTON) && (j != Sys->objet) &&
							((Sys->AdrObjet)[j].ob_state & SELECTED))
					{ /* Les mettre en normal si RBUTTON sauf l'objet cliqu. */
						(Sys->AdrObjet)[j].ob_state &= ~SELECTED;
    
						objc_draw (Sys->AdrObjet, j, MAX_DEPTH, 
											(Sys->AdrObjet)->ob_x, (Sys->AdrObjet)->ob_y,
											(Sys->AdrObjet)->ob_width, (Sys->AdrObjet)->ob_height);
					}
					j = (Sys->AdrObjet)[j].ob_next; 				/* Au suivant...     */
				} while ((j <= i) && (j > (Sys->AdrObjet)[i].ob_next));
			}
		}
	}
}

/*-----------------------------------------------------------------------*
 * Gestion des evenements clavier                                        *
 *                                                                       *
 * Derniere modification : 17/05/1996                                    *
 *-----------------------------------------------------------------------*/
static void gere_clavier(void)
{
	int evnt = MU_KEYBD, i;
	char ctr = 0, option[50];
	unsigned char touc;
	int top, dummy;
	Wind *wind;
	WindForm *ptr_dial;
	int cur_pos, cur_objc, dialog = TRUE;
	char mes[60];

	/* On regarde s'il ne s'agit pas d'un evenement special */
	EvntSpe (&evnt);

	/* Recupere  l'etat des touches speciales */
	Sys->kbd = (int)Kbshift (BLANK);
	/* Annuler bit CapsLock           */
	Sys->kbd &= ~0x10;								 	 
	if ((Sys->kbd == K_RSHIFT) || (Sys->kbd == K_LSHIFT))
		ctr = 0x01;					 				 /* Car. reprsentant la touche spciale */
	else if (Sys->kbd == K_CTRL)
		ctr = 0x05E;
	else if (Sys->kbd == K_ALT)
		ctr = 0x07;
	else
		ctr = ZERO;

	wind_get (ZERO, WF_TOP, &top, &dummy, &dummy, &dummy);

	wind = TopListe(top);
  
 	if (wind != WIND_NULL)
	{
		/* Si fenetre iconifiee, on oublie l'evenement */
		if (wind->iconified)
			return;

		/* Evenements speciaux pour les fentres images et textes */
		if (evnt != MU_KEYBD)
		{
			switch (wind->type)
			{
				case WTYPTEXT :
					WindTextDo(wind, evnt);
					break;
			}
			if (wind->type != WTYPFORM)
				return;
		}
		if (wind->type == WTYPFORM)
		{
			ptr_dial			= wind->cont.dialog;
			Sys->AdrObjet = ptr_dial->adr_form;
			Sys->objet		= ptr_dial->edit_objc;
			Sys->new_objc	= Sys->objet;
			Sys->pos			= ptr_dial->edit_pos;
			Sys->new_pos	= Sys->pos;
		}
		else
			return;
	}
	else
	{
		stdkey (&touc); 										/* Recherche code Ascii        */
		i = ZERO;
		do																	/* Pour chaque objet du menu   */
		{
			if (AdrMenu[i].ob_type == G_STRING)				/* Si c'est une option */
			{
				strcpy (option, AdrMenu[i].ob_spec.free_string);		/* La lire */
				trim (option);														/* Virer les espaces */
				if ((*(option + strlen (option) - 1) == touc) && (*(option + strlen (option) - 2) == ctr))
				{ /* Si le caractere et la touche speciale correspondent */
					if (NOT (AdrMenu[i].ob_state & DISABLED))			 /* Si actif */
					{	/* Generer message de selection d'une option du menu */
						EnvoiMessage(MN_SELECTED, m_title(AdrMenu, i), i);
					}
				}
			}
		} while (NOT (AdrMenu[i++].ob_flags & LASTOB));
		return;
	}

	if ((Sys->key == RETURN) || (Sys->key == ENTER)) 
	{	/* Si <Return> ou <Enter>, chercher bouton DEFAULT s'il y en a */
		i = ZERO;
		do
		{
			if (Sys->AdrObjet[i].ob_flags & DEFAULT)
			{
				form_button(Sys->AdrObjet, i, 1, &(Sys->new_objc));
				Sys->objet = i;
				(*wind->fonct)(MU_BUTTON);
				return;
			}
		} while (NOT (Sys->AdrObjet[i++].ob_flags & LASTOB)); 
	}

	/* Interprete et traite l'evenement clavier */
	dialog = do_keybd(Sys->AdrObjet, Sys->objet);
	if (Sys->key)
	{
		objc_edit(Sys->AdrObjet, Sys->objet, Sys->key, Sys->pos, EDCHAR, &Sys->pos);
		Sys->new_objc = Sys->objet;
		Sys->new_pos = Sys->pos;
	}

	/* Repositionner le curseur */
	if (dialog > 0) 
	{
		if (Sys->new_objc > 0 && Sys->new_objc != Sys->objet || Sys->new_pos != Sys->pos)
		{
			objc_edit(Sys->AdrObjet, Sys->objet, 0, Sys->pos, EDEND, &Sys->pos);
			Sys->pos = Sys->new_pos;
			objc_edit(Sys->AdrObjet, Sys->new_objc, 0, Sys->pos, EDEND, &Sys->pos);
		}
	}
	/* On sauve les parametres dans l'objet WindForm */
	ptr_dial->edit_objc = Sys->new_objc;
	ptr_dial->edit_pos = Sys->pos;
	
	/* Traiter l'eventuel raccourcis clavier */
	if (dialog == -1)
	{
		(*wind->fonct)(MU_BUTTON);
		return;				
	}
}

static void EvntSpe (int *evnt)
{
	/* On definit s'il s'agit ou non d'un evenement special */

  if (Sys->key == HELP)
		*evnt = EV_HELP;
	else if (Sys->key == UNDO)
		*evnt = EV_UNDO;
	else if (Sys->key == ESC)
		*evnt = EV_ESC;
	else if (Sys->key == HOME)
		*evnt = EV_HOME;
	else if (Sys->key == SH_HOME)
		*evnt = EV_SHHOME;
	else if (Sys->key == ARUP)
		*evnt = EV_ARUP;
	else if (Sys->key == ARDN)
		*evnt = EV_ARDN;
	else if (Sys->key == ARLF)
		*evnt = EV_ARLF;
	else if (Sys->key == ARRT)
		*evnt = EV_ARRT;
	else if (Sys->key == SH_ARUP)
		*evnt = EV_SHARUP;
	else if (Sys->key == SH_ARDN)
		*evnt = EV_SHARDN;
	else if (Sys->key == SH_ARLF)
		*evnt = EV_SHARLF;
	else if (Sys->key == SH_ARRT)
		*evnt = EV_SHARRT;
}

/**************************-=< Fin du module >=-**************************/
