/************************************************************************
 *                                                                      *
 *  WindGem.H : Dfinitions utilises par les procdures WindGem.       *
 *                                                                      *
 ************************************************************************/

#ifndef __WINDGEM__
#define __WINDGEM__

/*-=< Section d'inclusion des fichiers d'entete necessaire >=------------*/
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <xgemfast.h>
#include <tosbind.h>
#include <atari.h>																				/* pour KEYTAB */
#include <malloc.h>

/*************************************************************************
 * Definitions utiles.                                                   *
 *************************************************************************/
#define min(A, B) ((A) < (B) ? (A) : (B))
#define max(A, B) ((A) > (B) ? (A) : (B))

#define TRUE  	1
#define FALSE 	0
#define FAIL	 -1
#define MAX_LEN 256
#define NOT 		!
#define BLANK  -1
#define ZERO 		0

/*************************************************************************
 * Types des fenetres.                                                   *
 *************************************************************************/
#define WTYPNONE -1						/* Pas de fenetre                          */
#define WTYPDESK 0						/* Type special pour la gestion            */
#define WTYPFORM 1						/* Fenetre de type WindForm                */
#define WTYPGRAP 2						/* Fenetre image                           */
#define WTYPTEXT 3						/* Fenetre texte                           */

/*************************************************************************
 * Dfinition des structures utilises par WindGem.                      *
 *************************************************************************/
typedef struct aide 					/* Structure de l'aide pour les Boutons    */
{
	int	 numObj;								/* Numro de l'objet pour l'aide           */
	char *txt;									/* Texte de l'aide                         */
	struct aide *suiv;					/* Aide suivante                           */
} AIDE;

typedef struct								/* Fenetre formulaire                      */
{
	OBJECT	*adr_form;					/* adresse du formulaire                   */
	int			offset;							/* Dcalage x et y pour boite "OUTLINED".	 */
	int			edit;								/* 1er champ editable ou BLANK.            */
	int			edit_objc;					/* Objet ditable courant.                 */
	int			edit_pos;						/* Position courante du curseur d'dition. */
	AIDE	  *aide;							/* Liste des messages d'aide               */
} WindForm;

typedef struct								/* Fenetre graphique                       */
{
	MFDB	*img;									/* Donnees image                           */
	int		nb_color;							/* Nombre de couleurs											 */
	int		full;									/* Flag pleine ouverture                   */
	int 	wtot;									/* Largeur maximale de l'image             */
	int		htot;									/* Hauteur maximale de l'image             */
	int		lin;									/* 1 ligne pour redraw                    */
	int 	col;									/* 1 colonne pour redraw                  */
} WindGraph;

typedef struct								/* Fenetre texte                           */
{
	char	**lignes;							/* Contenu de la fentre                   */
	int		full;									/* Flag de pleine ouverture                */
	int		nbrel;								/* Nbre de lignes du texte                 */
	int		nbrec;								/* Nbre de colonnes du texte               */
	int		lin;									/* 1 ligne de texte                       */
	int		col;									/* 1 colonne de texte                     */
} WindText;

typedef union cont						/* Definition du contenu d'une fenetre     */
{
	WindForm	*dialog;					/* Type boite de dialogue                  */
	WindGraph	*image;						/* Type fenetre image                      */
	WindText	*text;						/* Type fenetre texte                      */
} Cont;

typedef struct listeFen				/* Liste des fenetres                      */
{
	int		handle;								/* handle de la fenetre si ouverte         */
	int		numObj;								/* Numero de la fenetre                    */
	int 	type;									/* type de la fenetre                      */
	int		mode;									/* fenetre modale ou non                   */
	char	*w_title;							/* pointeur sur une chaine titre           */
	int Xpos, Ypos, Wpos, Hpos;	/* Position courante											 */
	int		iconified;						/* Flag iconification                      */
	Cont 	cont;									/* Contenu de la fenetre                   */
	void 	(*fonct)(int evnt);		/* fonction de gestion de la fenetre ou \0 */
} Wind;

#define WNORM		0							/* Fenetre normale                         */
#define WMODAL	1							/* Fenetre modale                          */

#define WINDFORM_NULL		(WindForm *)0
#define WINDGRAPH_NULL	(WindGraph *)0
#define WINDTEXT_NULL		(WindText *)0
#define WIND_NULL				(Wind *)0

#define FW_ATTRIB	(NAME|CLOSER|MOVER|SMALLER)
#define GW_ATTRIB (NAME|CLOSER|FULLER|MOVER|SMALLER|SIZER|UPARROW|DNARROW|VSLIDE|LFARROW|RTARROW|HSLIDE)
#define TW_ATTRIB (NAME|CLOSER|FULLER|MOVER|SMALLER|SIZER|UPARROW|DNARROW|VSLIDE|LFARROW|RTARROW|HSLIDE)

/*************************************************************************
 * Structure systeme.                                                    *
 *************************************************************************/
#define NBWIND	60						/* Nombre maximum de fenetres              */

typedef struct								/* Parametres systeme                      */
{
	int AppId;									/* Id application                          */
	int VdiHandle;							/* Handle Vdi                              */
	int mousex, mousey, mousek;	/* Informations souris                     */
	int key, kbd;								/* Gestion du clavier                      */
	int clik;										/* Clic souris                             */
	int color_3d1, color_3d2;		/* Couleur pour la 3D                      */
	int Wchar, Hchar;						/* Taille caractere                        */
	OBJECT *AdrUser;						/* Objet contenant les objets Userdefs     */
	OBJECT *AdrIcone;						/* Objet contenant l'icones pour iconify   */
	OBJECT *AdrObjet;						/* Adresse objet courant                   */
	OBJECT *AdrAide;						/* Adresse objet aide en ligne             */
	int popMenu;								/* Numro de l'arbre d'objets pour popup   */
	void (*popup)(int event);		/* Fonction de gestion du popup menu       */
	int Xdesk,Ydesk,Wdesk,Hdesk;/* Coordonnees du bureau                   */
	int Xres, Yres;							/* Resolution de l'ecran                   */
	int Nplane;									/* Nombre de plans du bureau               */
	int baseline;								/* Premiere ligne de texte dans une fenetre*/
	int objet, pos;							/* Gestion des objets bloquants            */ 
	int new_objc, new_pos;			/* Nouvel objet, nouvelle position         */
	int lastMode;								/* Mode de la fenetre actuelle (MODAL ?!)  */
	int lastImg;								/* Dernier numero de fenetre graphique     */
	int lastTxt;								/* Dernier numero de fenetre texte		     */
	void (*menu)(int option);		/* Fonction de gestion du menu             */
	void (*desk)(int event);		/* Fonction de gestion du bureau           */
} System;

/*************************************************************************
 * Valeur de definition des objets UserDefs.                             *
 *************************************************************************/
#define UNDER_B 18				/* Pour raccourcis clavier                     */
#define SMALL_B 22				/* Petite criture                             */
#define POPUP_B 24				/* Boutons PopUp                               */
#define RADIO_B 26				/* Radio-Boutons                               */
#define CHECK_B 28				/* Check-Box                                   */

/*************************************************************************
 * Definition des touches.                                               *
 *************************************************************************/
#define HELP		0x6200
#define UNDO		0x6100
#define ARUP		0x4800
#define ARDN		0x5000
#define ARLF		0x4B00
#define ARRT		0x4D00
#define SH_ARUP 0x4838
#define SH_ARDN 0x5032
#define SH_ARLF 0x4B34
#define SH_ARRT 0x4D36
#define ESC			0x011B
#define CLR			0x4700
#define RETURN	0x720D
#define ENTER		0x1C0D
#define HOME		0x4700
#define SH_HOME	0x4737

/*************************************************************************
 * Definition des nouveaux evenements Userdefs.                          *
 *************************************************************************/
#define EV_NO			 0		/* Pas d'evenement                               */
#define EV_OPEN		 8192	/* Ouverture Fenetre                             */
#define EV_CLOSE	-1		/* Fermeture Fenetre                             */
#define EV_HELP		-100	/* Touche Help                                   */
#define EV_UNDO		-101	/* Touche Undo                                   */
#define EV_ARUP		-102	/* Fleche Haut                                   */
#define EV_ARDN		-103	/* Fleche Bas                                    */
#define	EV_ARLF		-104	/* Fleche Gauche                                 */
#define EV_ARRT		-105	/* Fleche Droite                                 */
#define EV_SHARUP	-106	/*  Fleche Haut                                 */
#define EV_SHARDN	-107	/*  Fleche Bas                                  */
#define EV_SHARLF	-108	/*  Fleche Gauche                               */
#define EV_SHARRT	-109	/*  Fleche Droite                               */
#define EV_HOME		-110	/* Touche Home                                   */
#define EV_SHHOME	-111	/* Touche  Home                                 */
#define EV_ESC		-112	/* Touche Escape                                 */

/*************************************************************************
 * Messages d'erreurs.                                                                     *
 *************************************************************************/
#define WindOK				 0	/* Tout s'est bien passe                       */
#define WindErrNoRsc	-1	/* Fichier RSC manquant                        */

/*************************************************************************
 * Prototypes des Fonctions de Dialogue.                                 *
 *************************************************************************/

/*-=< Routines UserDefs >=-----------------------------------------------*/
void set_user(OBJECT *addr_tree);

/*-=< Gestion de l'application >=----------------------------------------*/
void AppInit(char *fic_rsc, int desk, void (*gereDesk)(int event), int menu, void (*gereMenu)(int event));
void AppExit(void);
void PopMenu(int numObj, void (*gerePopup)(int option));

/*-=< Gestion des formulaires - GemForm.c >=-----------------------------*/
void WindFormInit(int numObj, int mode, int inf_x, int inf_y, char *title, int edit, void (*fonct)(int evnt));
void WindOpen(int numObj);
void WindClose(int numObj);
void WindDelete(int numObj);
void ObjcUnselect(int arbre, int numObj);
void WindDraw(int arbre, int objet);

int GetObjet(void);
int EventMulti (void);

/*-=< Gestion de l'aide - GemHelp.c >=-----------------------------------*/
void AjouteAide (int arbre, int objet, char *txt);

/*-=< Gestion des images - GemGrap.c >=----------------------------------*/
int WindGraphInit(MFDB *image, char *title, int x, int y, int w, int h, int nb_col, int max_x, int max_y);

/*-=< Gestion des popup - GemPopUp.c >=----------------------------------*/
void pop_up (int arbre, int obj, int pu);

/*-=< Gestion des textes - GemText.c >=----------------------------------*/
int WindTextInit (char *file, GRECT *Fpos);

/*-=< Fonctions diverse : GemUtil.h >=-----------------------------------*/
#define INS_AVANT   -1
#define INS_APRES    1

void FormAlerte(int index);
void get_bkgr (int of_x, int of_y, int of_w, int of_h, MFDB *img);
void put_bkgr (int of_x, int of_y, int of_w, int of_h, MFDB *img);

int parent (OBJECT *adr, int object);
void set_clip (int clip_flag, GRECT *area);
void EnvoiMessage(int type, int handle, int message);
char *trim (char *str);
char *strnpcpy(char *dest, char *start, char *stop);
char *get_text (OBJECT *adr, int object);
void set_text (OBJECT *adr, int object, char *string);
long exist (char *name, int att);
char *path (char *chemin);
int selector (char *chemin, char *ext, char *file, char *title);
void extension (char *filename, char *ext);
void litdate (char *dat, unsigned int fdate);
void convdate (char *date, unsigned int *fdate);
char *strnpcpy(char *dest, char *start, char *stop);
char *strcomplete (char *chaine, int lng, char car, int sens);
char *strinsert (char *chaine, char *ch);
char *strcopy(char *res, char *chaine, int deb, int fin);
void cre_fichier(char *chemin, char *fic, char *fichier);

#endif

/*************************************************************************
 * Fin du module de declaration.                                         *
 *************************************************************************/
